#-*- coding: utf-8 -*-   
import argparse
import sys
from cli import CLI
import import_ipynb
import matplotlib.pyplot as plt
import config
import numpy as np

def main():

    parser = argparse.ArgumentParser(
        description='Python package for analysis in the Muse and Pupil devices.',
        usage=''' __main__ <command> [<args>]
        Input
                      Analyzabile folder 
        folder        input folder (with files inside)
                      output folder
        
        Available commands:
        devices        Analyzable devices
                        -m --muse        EEG data
                        add channel to analyze 
                                    TP9 AF7 AF8 TP10
                        -p --ppg         Include PPG data
                        add channel to analyze 
                                    PPG1 PPG2 PPG3 
                        -c --acc         Include accelerometer data
                        add channel to analyze
                                    X Y
                                    
                        -g --gyro        Include gyroscope data
                        add channel to analyze 
                                    Y Z
        
        P.S.: if a channel isn't inserted, all will be analyzed
                        ''')
    parser.add_argument('inputfoldername')
    parser.add_argument('outputfoldername')
    parser.add_argument('command', help='Command to run.')
    

    # parse_args defaults to [1:] for args, but you need to
    # exclude the rest of the args too, or validation will fail
    
    args = parser.parse_args(sys.argv[1:4])
    global input_folder 
    input_folder=args.inputfoldername
    global output_folder
    output_folder=args.outputfoldername
    if not hasattr(CLI, args.command):
        print('Incorrect usage. See help below.')
        parser.print_help()
        exit(1)

    cli = CLI(args.command)
def analyze(muse=None,pupil=None,ppg=False,acc=False,gyro=False,):
    if muse:
        file=find('EEG*', input_folder)
        file=(', '.join(file))
        if "all" in muse:
            muse=[0,1,2,3,4]
        else:
            muse=(["timestamps"]+muse)
        from EEG import EEG
        df_EEG=EEG.EEG(file,muse,output_folder)
        violin_plot(df_EEG,"EEG_")
        #violin plot for different band 
        m=df_EEG.columns.str.contains(' alpha')| df_EEG.columns.str.contains(' beta')| df_EEG.columns.str.contains(' theta')| df_EEG.columns.str.contains(' delta')
        violin_plot(df_EEG[df_EEG.columns[m]],"EEG_band_")
        
        #violin plot for mental metrics
        m=df_EEG.columns.str.contains('fatigue')| df_EEG.columns.str.contains('relaxation')| df_EEG.columns.str.contains('concentration')
        violin_plot(df_EEG[df_EEG.columns[m]],"EEG_mental_metrics_")
       
        #violin plot for non-linear
        m=df_EEG.columns.str.contains('DFA')| df_EEG.columns.str.contains('PFD')| df_EEG.columns.str.contains('Entropy')| df_EEG.columns.str.contains('Hurst')
        violin_plot(df_EEG[df_EEG.columns[m]],"EEG_noLinear_")
        
        df_EEG.to_pickle(output_folder+"/df_EEG.pkl")
    
    if ppg:
        file=find('PPG*', input_folder)
        file=(', '.join(file))
        if "all" in ppg:
            ppg=["PPG1","PPG2","PPG3"]
        else:
            ppg=ppg
        from PPG import PPG
        df_PPG=PPG.PPG(file,ppg,output_folder)
        violin_plot(df_PPG,"PPG")
        df_PPG.to_pickle(output_folder+"/df_PPG.pkl")
    if acc:
        file=find('ACC*', input_folder)
        file=(', '.join(file))
        if "all" in acc:
            acc=[0,1,2]
        else:
            acc=(["timestamps"]+acc)
        from ACC import ACC
        df_ACC=ACC.ACC(file,acc,output_folder)
        plt.clf()
        plt.figure(figsize=(20,10))
        plt.plot(df_ACC["axis_X"], label="Sagittal Axis",linewidth=5)
        plt.plot(df_ACC["axis_Y"],label="Lateral Axis",linewidth=5)
        plt.legend(loc="upper right")
        plt.savefig(output_folder+"/acc.jpg")
        
        violin_plot(df_ACC,"ACC")
        df_ACC.to_pickle(output_folder+"/df_ACC.pkl")
    if gyro:
        file=find('GYRO*', input_folder)
        file=(', '.join(file))
        if "all" in gyro:
            gyro=[0,1,2,3]
        else:
            gyro=(["timestamps"]+gyro)
        from GYRO import GYRO
        df_GYRO=GYRO.GYRO(file,gyro,output_folder) 
        violin_plot(df_GYRO,"GYRO")
        df_GYRO.to_pickle(output_folder+"/df_GYRO.pkl")
        
        plt.clf()
        fig, ax = plt.subplots(3,1,figsize=(20,10))
        ax[0].plot(df_GYRO["head-movement-vertical"], label="Head movement vertical",linewidth=5)
        ax[0].legend(['Head movement vertical'])
        ax[1].plot(df_GYRO["head-movement-horizontal"],label="Head movement horizontal",linewidth=5)
        ax[1].legend(['Head movement horizontal'])
        ax[2].plot(df_GYRO["movement"],label="Movement",linewidth=5)
        ax[2].legend(['Movement'])
        plt.legend(loc="upper right")
        plt.savefig(output_folder+"/gyro.jpg")

def violin_plot(df,title):
    fig, ax = plt.subplots(nrows=1,ncols=(len(df.columns)),figsize=(35,25),constrained_layout=True)

    #fig.subplots_adjust(wspace=2)
    #fig.tight_layout()
    for i in range(df.shape[1]):
        ax[i].violinplot(df[df.columns[i]],showmedians=True)
        title_column=pretty_name(df.columns[i])
        ax[i].set_title(title_column,fontsize=30)
    plt.savefig(output_folder+"/"+title+"violin.jpg")

def pretty_name(text):
    words = text.split("_")
    total_string = ""
    for counter, word in enumerate(words):
        if counter>0 and counter % 1 == 0:
            total_string +="\n{}".format(word)
        else:
            total_string +=" {}".format(word)
    return total_string.lstrip()

import os, fnmatch
def find(pattern, path):
    result = []
    for root, dirs, files in os.walk(path):
        for name in files:
            if fnmatch.fnmatch(name, pattern):
                result.append(os.path.join(root, name))
    return result


    
if __name__ == '__main__':
    main()